import bpy
import blf
import logging
from mathutils import Vector
from bpy_extras.view3d_utils import location_3d_to_region_2d
import math
from collections import OrderedDict
import re  # 引入正则表达式模块
from bpy.app.translations import pgettext_iface as _  # 使用 Blender 内置的翻译函数
from functools import partial

# 设置日志记录
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

# 定义一个唯一的 owner 标识，用于 msgbus 订阅
GRID_DIMENSION_OWNER = "grid_dimension_owner"

# 全局存储每个场景的之前单位系统
previous_unit_system = {}

# 定义网格层级配置
GRID_LEVELS = {
    # 公制单位
    'metric': {
        'meter': {
            'main_interval': 1.0,  # 1m
            'max_distance': 10.0,  # 10m视距内显示
            'format': lambda x: f"{x:g}m"  # 使用 g 格式去除不必要的小数点和0
        },
        'decameter': {
            'main_interval': 10.0,  # 10m
            'max_distance': 100.0,
            'format': lambda x: f"{x:g}m"
        },
        'hectometer': {
            'main_interval': 100.0,  # 100m
            'max_distance': 1000.0,
            'format': lambda x: f"{x:g}m"
        },
        'kilometer': {
            'main_interval': 1000.0,  # 1km
            'max_distance': 10000.0,
            'format': lambda x: f"{x/1000:g}km"
        }
    },
    # 英制单位 - 调整为与 Blender 网格对齐
    'imperial': {
        'foot': {
            'main_interval': 0.3048,  # 1ft in meters
            'max_distance': 3.048,    # 10ft
            'format': lambda x: f"{x/0.3048:.0f}'"  # 转换为英尺并显示
        },
        'ten_feet': {
            'main_interval': 3.048,   # 10ft
            'max_distance': 30.48,    # 100ft
            'format': lambda x: f"{x/0.3048:.0f}'"
        },
        'hundred_feet': {
            'main_interval': 30.48,   # 100ft
            'max_distance': 304.8,    # 1000ft
            'format': lambda x: f"{(x/0.3048):.0f}'"
        },
        'thousand_feet': {
            'main_interval': 304.8,   # 1000ft
            'max_distance': 3048.0,   # 10000ft
            'format': lambda x: f"{(x/0.3048/1000):.1f}k'"
        }
    }
}

# 统一的子刻度比例
SUB_DIVISIONS = [0.2, 0.4, 0.6, 0.8]  # 20%, 40%, 60%, 80%

def convert_to_imperial(value_in_meters):
    """将米转换为英尺"""
    return value_in_meters * 3.28084

def convert_to_metric(value_in_feet):
    """将英尺转换为米"""
    return value_in_feet * 0.3048

class GridDimensionSettings(bpy.types.PropertyGroup):
    follow_scene_units: bpy.props.BoolProperty(
        name=_("Units"),
        description=_("Automatically follow the scene's unit system for grid dimensions."),
        default=True,
        update=lambda self, context: update_overlay()
    )

    unit_system: bpy.props.EnumProperty(
        name=_("Unit System"),
        description=_("Choose the unit system for grid dimensions."),
        items=[
            ('m', _("Meters"), _("Meters")),
            ('ft', _("Feet"), _("Feet")),
        ],
        default='m',
        update=lambda self, context: update_overlay()
    )

    color: bpy.props.FloatVectorProperty(
        name=_("Color"),
        subtype='COLOR',
        default=(1.0, 1.0, 1.0),
        min=0.0,
        max=1.0,
        update=lambda self, context: update_overlay()
    )
    alpha: bpy.props.FloatProperty(
        name=_("Alpha"),
        default=0.75,
        min=0.0,
        max=1.0,
        subtype='FACTOR',  # 使用进度条样式显示
        update=lambda self, context: update_overlay()
    )
    font_size: bpy.props.IntProperty(
        name=_("Font Size"),
        default=12,
        min=1,
        update=lambda self, context: update_overlay()
    )
    
    show_z: bpy.props.BoolProperty(
        name=_("Z-axis"),
        description=_("Enable to display Z axis grid dimensions in perspective view."),
        default=False,
        update=lambda self, context: update_overlay()
    )
    
    auto_color: bpy.props.FloatProperty(
        name=_("Axis Color"),
        description=_("Mix between custom color (0.0) and axis color (1.0)"),
        default=0.0,
        min=0.0,
        max=1.0,
        subtype='FACTOR',  # 使用进度条样式显示
        update=lambda self, context: update_overlay()
    )
    
    show_count: bpy.props.BoolProperty(
        name=_("Count"),
        description=_("Display the number of annotations in the bottom right corner"),
        default=False,
        update=lambda self, context: update_overlay()
    )

class GridDimensionOverlay:
    overlays = {}
    
    def __init__(self, area):
        self.area = area
        self.handle = None
        self.last_positions = {'X': OrderedDict(), 'Y': OrderedDict(), 'Z': OrderedDict()}
        self.position_memory = 0.9
        self.last_view_rotation = None  # 存储上一次的视图旋转
        self.rotation_tolerance = 0.1  # 旋转宽容度（弧度）
        self.color = (1.0, 1.0, 1.0)
        self.alpha = 1.0
        self.font_size = 12
        self.grid_scale = 1.0
        self.active = False
        self.annotation_count = 0  # 添加标注计数器
        self.unit_system = 'm'  # 添加单位系统
        
    def register(self, report_func):
        if self.handle is None:
            if self.area.as_pointer() not in GridDimensionOverlay.overlays:
                logger.info(f'Registering overlay for area: {self.area.as_pointer()}')
                window_region = next((region for region in self.area.regions if region.type == 'WINDOW'), None)
                if window_region is None:
                    logger.error(f'WINDOW region not found for area: {self.area.as_pointer()}')
                    report_func({'ERROR'}, _("Failed to find WINDOW region, cannot register grid dimension."))
                    return
                try:
                    # 使用 functools.partial 绑定 self
                    draw_func = partial(self.draw_callback_px, self)
                    self.handle = bpy.types.SpaceView3D.draw_handler_add(
                        draw_func, 
                        (),  # 不传递额外参数
                        'WINDOW', 
                        'POST_PIXEL'
                    )
                    GridDimensionOverlay.overlays[self.area.as_pointer()] = self
                    self.active = True
                    logger.info(f'Successfully registered overlay for area: {self.area.as_pointer()}')
                    self.area.tag_redraw()  # 强制区域重绘
                except Exception as e:
                    logger.exception("Failed to add draw handler.")
                    report_func({'ERROR'}, _("Error registering grid dimension: ") + str(e))
            else:
                logger.warning(f'Overlay already registered for area: {self.area.as_pointer()}')
                report_func({'WARNING'}, _("Overlay already registered for this area."))
    
    def unregister(self):
        if self.handle is not None:
            try:
                bpy.types.SpaceView3D.draw_handler_remove(self.handle, 'WINDOW')
                logger.info(f'Unregistered overlay for area: {self.area.as_pointer()}')
                self.area.tag_redraw()  # 强制区域重绘
            except Exception as e:
                logger.exception("Failed to remove draw handler.")
            self.handle = None
            self.active = False
            GridDimensionOverlay.overlays.pop(self.area.as_pointer(), None)
    
    @staticmethod
    def is_overlay_active(area):
        active = area.as_pointer() in GridDimensionOverlay.overlays
        logger.debug(f'Checking if overlay is active for area: {area.as_pointer()}, Result: {active}')
        return active
    
    def update_settings(self, scene):
        """更新所有设置"""
        settings = scene.grid_dimension  # 直接访问 PropertyGroup
        self.color = settings.color
        self.alpha = settings.alpha
        self.font_size = settings.font_size
        self.grid_scale = scene.unit_settings.scale_length
        
        # 更新单位系统
        if settings.follow_scene_units:
            self.unit_system = 'ft' if scene.unit_settings.system == 'IMPERIAL' else 'm'
        else:
            self.unit_system = settings.unit_system

    def draw_callback_px(self, overlay):
        """绘制回调函数"""
        try:
            # 如果全局overlays关闭，则不绘制任何内容
            if not bpy.context.space_data.overlay.show_overlays:
                return
                
            context = bpy.context
            scene = context.scene
            
            # 检查当前区域是否为目标区域
            current_area = bpy.context.area
            if current_area is None or current_area.as_pointer() != self.area.as_pointer():
                return

            self.update_settings(scene)

            # 获取WINDOW区域
            window_region = next((region for region in self.area.regions if region.type == 'WINDOW'), None)
            if window_region is None:
                logger.error(f'WINDOW region not found during draw for area: {self.area.as_pointer()}')
                return

            region = window_region
            space = self.area.spaces[0]
            rv3d = space.region_3d

            # 确定当前视图方向
            view_orientation = self.get_view_orientation(rv3d)
            logger.debug(f'Current view orientation: {view_orientation}')

            # 根据视图方向决定显示哪些轴
            axes_to_show = {'X', 'Y', 'Z'}  # 默认显示所有轴
            view_orientation = self.get_view_orientation(rv3d)
            
            if rv3d.view_perspective == 'ORTHO':
                # 正交视图下根据视图方向显示对应轴
                # 添加视角宽容度，允许轻微偏移
                view_tolerance = 0.2  # 约11.5度的宽容度
                
                # 计算视图向量与标准方向的点积
                view_vector = rv3d.view_rotation @ Vector((0, 0, -1))
                view_vector.normalize()
                
                front_back = abs(abs(view_vector.y) - 1) < view_tolerance
                left_right = abs(abs(view_vector.x) - 1) < view_tolerance
                top_bottom = abs(abs(view_vector.z) - 1) < view_tolerance
                
                if front_back:
                    axes_to_show = {'X', 'Z'}
                elif left_right:
                    axes_to_show = {'Y', 'Z'}
                elif top_bottom:
                    axes_to_show = {'X', 'Y'}
                else:
                    # 用户视图下，显示所有可见轴
                    axis_visibility = {
                        'X': abs(view_vector.x),
                        'Y': abs(view_vector.y),
                        'Z': abs(view_vector.z)
                    }
                    
                    # 如果轴向量与视图方向夹角接近90度（即垂直），则显示该轴
                    # 0.9对应约25度的夹角范围
                    axes_to_show = {axis for axis, vis in axis_visibility.items() if vis < 0.9}
                    
                    # 确保至少显示两个最垂直的轴
                    if len(axes_to_show) < 2:
                        sorted_axes = sorted(axis_visibility.items(), key=lambda x: x[1])
                        axes_to_show = {sorted_axes[0][0], sorted_axes[1][0]}
                
                # 在正交视图下也应用 Show Z 设置
                if not scene.grid_dimension.show_z:
                    axes_to_show.discard('Z')
            
            elif rv3d.view_perspective == 'PERSP':
                if not scene.grid_dimension.show_z:
                    axes_to_show.discard('Z')

            # 获取视距和相机位置
            view_distance = rv3d.view_distance
            camera_location = rv3d.view_matrix.inverted().translation

            # 定义轴向颜色
            axis_colors = {
                'X': (1.0, 0.0, 0.0),  # 红色
                'Y': (0.0, 1.0, 0.0),  # 绿色
                'Z': (0.0, 0.0, 1.0)   # 蓝色
            }

            # 批量绘制文本
            blf_batch = []
            total_annotations = 0
            origin_added = False  # 跟踪是否已添加原点

            # 首先检查是否应该显示原点
            if rv3d.view_perspective == 'ORTHO':
                bounds = self.get_ortho_view_bounds(region, rv3d)
            else:
                bounds = self.get_view_bounds(region, rv3d)
            
            # 只在透视模式下显示原点
            if rv3d.view_perspective == 'PERSP' and bounds['min_x'] <= 0 <= bounds['max_x'] and bounds['min_y'] <= 0 <= bounds['max_y']:
                # 将3D坐标转换为2D屏幕坐标
                origin_pos_2d = location_3d_to_region_2d(region, rv3d, Vector((0, 0, 0)))
                if origin_pos_2d is not None:
                    # 检查是否在屏幕范围内
                    if (0 <= origin_pos_2d.x <= region.width and 
                        0 <= origin_pos_2d.y <= region.height):
                        # 添加原点标注，整体上移10个像素
                        blf_batch.append((
                            origin_pos_2d.x,
                            origin_pos_2d.y + 10,
                            "0",
                            0.8,  # 完全不透明
                            self.font_size * 1.0,  # 稍大的字体
                            (0.9, 0.9, 0.9),  # 特殊的原点颜色
                            True  # 标记为原点
                        ))
                        total_annotations += 1
                        origin_added = True

            # 处理每个轴的标注
            self.annotation_count = 0
            
            for axis in axes_to_show:
                # 生成该轴的所有网格点
                if rv3d.view_perspective == 'ORTHO':
                    grid_points = self.generate_ortho_grid_points(view_distance, axis, region, rv3d)
                else:
                    grid_points = self.generate_grid_points(camera_location, view_distance * 2, axis, region, rv3d)
                
                self.annotation_count += len(grid_points)  # 更新计数
                
                for point in grid_points:
                    try:
                        # 跳过原点（因为我们已经单独处理了）
                        if abs(point['value']) < 1e-6:
                            continue
                        
                        # 将3D坐标转换为2D屏幕坐标
                        pos_2d = location_3d_to_region_2d(region, rv3d, point['position'])
                        if pos_2d is None:
                            continue

                        # 检查是否在屏幕范围内
                        if not (0 <= pos_2d.x <= region.width and 0 <= pos_2d.y <= region.height):
                            continue

                        # 计算淡出因子
                        fade_factor = self.calculate_fade_factor(point, view_distance)
                        
                        if fade_factor < 0:
                            continue

                        # 计算缩放因子
                        scale_factor = self.calculate_distance_scale(point['group_distance'], view_distance)
                        if point['is_sub']:
                            scale_factor *= 0.8  # 子刻度额外缩小20%

                        text = point['format'](point['value'])
                        
                        axis_color = axis_colors[axis]
                        mix_factor = scene.grid_dimension.auto_color
                        if mix_factor < 1:  # 当不是完全使用轴颜色时
                            axis_color = (
                                self.color[0] * (1 - mix_factor) + axis_color[0] * mix_factor,
                                self.color[1] * (1 - mix_factor) + axis_color[1] * mix_factor,
                                self.color[2] * (1 - mix_factor) + axis_color[2] * mix_factor
                            )

                        # 使用当前字号作为基准进行缩放
                        base_size = self.font_size if not point['is_sub'] else int(self.font_size * 0.8)
                        final_size = max(int(base_size * 0.6), int(base_size * scale_factor))  # 确保不会缩得太小

                        # 计算边缘淡出
                        margin = 60  # 边缘淡出区域的像素宽度
                        edge_factor = min(
                            pos_2d.x / margin,
                            (region.width - pos_2d.x) / margin,
                            pos_2d.y / margin,
                            (region.height - pos_2d.y) / margin
                        )
                        edge_factor = max(0.0, min(1.0, edge_factor))
                        
                        # 应用边缘淡出到过渡因子
                        transition_factor = point['transition_factor'] * edge_factor
                        
                        # 计算最终透明度
                        final_alpha = fade_factor * self.alpha * transition_factor
                        # 在正交视图下增加透明度
                        if rv3d.view_perspective == 'ORTHO':
                            final_alpha = min(1.0, final_alpha * 1.5)  # 增加50%的透明度，但不超过1.0
                        
                        # 只有透明度大于0.01的标注才计入总数
                        if final_alpha > 0.01:
                            total_annotations += 1
                        
                        # 添加到绘制批次，整体上移10个像素
                        blf_batch.append((
                            pos_2d.x, pos_2d.y + 10, text,
                            final_alpha, final_size, axis_color, False))

                    except Exception as e:
                        logger.exception(f'Error processing grid point: {e}')

            # 批量绘制文本
            for x, y, text, alpha, font_size, color, is_origin in blf_batch:
                try:
                    # 特殊处理原点标注
                    if is_origin:
                        # 原点使用稍大的字体和特殊颜色
                        blf.size(0, int(font_size * 1.2))
                        origin_color = (0.9, 0.9, 0.9)  # 接近白色
                        blf.color(0, origin_color[0], origin_color[1], origin_color[2], alpha)
                    else:
                        blf.size(0, font_size)
                        blf.color(0, color[0], color[1], color[2], alpha)
                    
                    # 获取文本尺寸以进行居中对齐
                    text_width, text_height = blf.dimensions(0, text)
                    
                    # 调整位置使文本居中
                    x = x - text_width / 2
                    y = y + text_height / 2  # 向上偏移以给点标记留出空间
                    
                    # 为原点添加阴影效果
                    if is_origin:
                        blf.enable(0, blf.SHADOW)
                        blf.shadow(0, 3, 0.0, 0.0, 0.0, alpha * 0.8)
                    
                    # 绘制文本
                    blf.position(0, x, y, 0)
                    blf.draw(0, text)
                    
                    if is_origin:
                        blf.disable(0, blf.SHADOW)
                    
                    # 在数字下方绘制点标记
                    if not is_origin:  # 原点不需要点标记
                        dot_y = y - text_height - 2  # 点标记位置在文本下方
                        blf.size(0, max(6, int(font_size * 0.5)))  # 点的大小
                        blf.position(0, x + text_width/2 - 1, dot_y, 0)  # 居中对齐点标记
                        blf.draw(0, "•")
                        
                except Exception as e:
                    logger.exception(f'Error drawing text: {e}')

            # 在绘制完所有标注后，独立显示计数（如果开启）
            if scene.grid_dimension.show_count:
                margin = 10
                blf.size(0, self.font_size)
                text = f"Count: {total_annotations}"
                text_width, text_height = blf.dimensions(0, text)
                x = region.width - text_width - margin
                y = margin + text_height
                self.draw_text(x, y, text, 1.0, self.font_size, self.color, False)

            # 在正交视图下为世界中心添加独立的0标注，并结合边缘淡出效果
            if rv3d.view_perspective == 'ORTHO':
                zero_pos = Vector((0, 0, 0))
                zero_pos_2d = location_3d_to_region_2d(region, rv3d, zero_pos)
                if zero_pos_2d:
                    margin = 50  # 边缘淡出区域的像素宽度
                    zero_edge_factor = min(
                        zero_pos_2d.x / margin,
                        (region.width - zero_pos_2d.x) / margin,
                        zero_pos_2d.y / margin,
                        (region.height - zero_pos_2d.y) / margin
                    )
                    zero_edge_factor = max(0.0, min(1.0, zero_edge_factor))
                else:
                    zero_edge_factor = 0.0
                
                zero_point = {
                    'position': zero_pos,
                    'level': 'zero',
                    'is_sub': False,
                    'distance': view_distance,
                    'value': 0.0,
                    'format': "{:.1f}",
                    'interval': 1.0,
                    'group_distance': view_distance,
                    'transition_factor': 1.0 * zero_edge_factor  # 结合边缘淡出效果
                }
                self.draw_text(zero_pos_2d.x, zero_pos_2d.y + 10, "0", zero_point['transition_factor'], self.font_size, self.color, True)
        
        except Exception as e:
            logger.exception(f'Error in draw_callback_px: {e}')

    def calculate_distance_scale(self, distance, view_distance):
        """计算基于距离的缩放因子"""
        # 调整缩放计算方法
        scale = 1.0 - (distance / (view_distance * 3)) * 0.4  # 减小距离影响，最多缩小40%
        return max(0.6, min(1.0, scale))  # 确保不会缩得太小

    def calculate_fade_factor(self, point, view_distance):
        """计算淡入淡出因子"""
        # 使用组距离来计算淡出效果
        group_distance = point['group_distance']
        
        # 降低距离衰减的敏感度
        distance_factor = 1.0 - (group_distance / (view_distance * 4))  # 从2改为4，减缓衰减速度
        
        # 使用平方根函数使过渡更加平缓
        if distance_factor > 0:
            distance_factor = math.sqrt(distance_factor)
        
        # 应用层级过渡因子
        distance_factor *= point['transition_factor']
        
        # 根据层级调整透明度
        if point['is_sub']:
            # 调整子刻度的淡出参数
            sub_fade_threshold = 0.9  # 提高阈值，让子刻度更晚开始淡出
            sub_fade_range = 0.6  # 增加范围，使淡出更加平缓
            
            if distance_factor < sub_fade_threshold:
                sub_factor = (distance_factor - (sub_fade_threshold - sub_fade_range)) / sub_fade_range
                sub_factor = max(0.0, min(1.0, sub_factor))
                # 使用平方根函数使子刻度的淡出更加平缓
                if sub_factor > 0:
                    sub_factor = math.sqrt(sub_factor)
                distance_factor *= sub_factor
        
        return max(0.0, min(1.0, distance_factor))

    def generate_grid_points(self, camera_location, view_distance, axis, region, rv3d):
        """生成多层级的网格点"""
        points = []
        
        # 如果是正交视图，直接返回空列表
        if rv3d.view_perspective == 'ORTHO':
            return points
            
        used_positions = set()  # 用于跟踪已使用的位置
        
        # 获取视口边界
        bounds = self.get_view_bounds(region, rv3d)
        
        # 获取场景的网格比例和单位系统
        scene = bpy.context.scene
        grid_scale = scene.unit_settings.scale_length
        
        # 选择当前单位系统的层级
        current_system = 'metric' if self.unit_system == 'm' else 'imperial'
        current_levels = GRID_LEVELS[current_system]
        
        # 根据轴选择对应的边界
        if axis == 'X':
            min_bound = bounds['min_x']
            max_bound = bounds['max_x']
        elif axis == 'Y':
            min_bound = bounds['min_y']
            max_bound = bounds['max_y']
        else:  # Z轴
            min_bound = bounds['min_z']
            max_bound = bounds['max_z']
        
        # 计算每个层级的过渡因子
        active_levels = []
        transition_factor = {}
        
        for level_name, level_data in current_levels.items():
            # 应用网格比例
            scaled_max_distance = level_data['max_distance'] * grid_scale
            scaled_interval = level_data['main_interval'] * grid_scale
            
            transition_range = scaled_max_distance * 0.5
            
            if view_distance <= scaled_max_distance + transition_range:
                if view_distance > scaled_max_distance:
                    factor = 1.0 - (view_distance - scaled_max_distance) / transition_range
                    factor = math.pow(factor, 2)
                else:
                    factor = 1.0
                
                scaled_data = level_data.copy()
                scaled_data['main_interval'] = scaled_interval
                scaled_data['max_distance'] = scaled_max_distance
                
                active_levels.append((level_name, scaled_data))
                transition_factor[level_name] = factor

        # 根据轴选择对应的边界
        if axis == 'X':
            min_bound = bounds['min_x']
            max_bound = bounds['max_x']
        elif axis == 'Y':
            min_bound = bounds['min_y']
            max_bound = bounds['max_y']
        else:  # Z轴
            min_bound = bounds['min_z']
            max_bound = bounds['max_z']
        
        # 扩展边界以确保覆盖完整的网格线
        bound_margin = (max_bound - min_bound) * 0.1  # 添加10%的边界余量
        min_bound -= bound_margin
        max_bound += bound_margin

        # 临时存储所有点，按组组织
        all_points = []
        
        # 为每个活动层级生成点
        for level_name, level_data in active_levels:
            interval = level_data['main_interval']
            # 使用视口边界来确定范围
            min_coord = math.floor(min_bound / interval) * interval
            max_coord = math.ceil(max_bound / interval) * interval
            
            # 生成主刻度和它们的子刻度组
            for coord in self.frange(min_coord, max_coord + interval, interval):
                grid_group = []
                pos = Vector((coord, 0, 0)) if axis == 'X' else \
                      Vector((0, coord, 0)) if axis == 'Y' else \
                      Vector((0, 0, coord))
                
                # 计算到相机的距离
                distance = (pos - camera_location).length
                
                main_point = {
                    'position': pos,
                    'level': level_name,
                    'is_sub': False,
                    'distance': distance,
                    'value': coord,
                    'format': level_data['format'],
                    'interval': interval,
                    'group_distance': distance,  # 组的距离使用主刻度的距离
                    'transition_factor': transition_factor[level_name]  # 添加过渡因子
                }
                grid_group.append(main_point)
                
                # 生成子刻度
                next_coord = coord + interval
                sub_points = []
                for ratio in SUB_DIVISIONS:
                    sub_coord = coord + (next_coord - coord) * ratio
                    if min_bound <= sub_coord <= max_bound:
                        sub_pos = Vector((sub_coord, 0, 0)) if axis == 'X' else \
                                Vector((0, sub_coord, 0)) if axis == 'Y' else \
                                Vector((0, 0, sub_coord))
                        
                        sub_point = {
                            'position': sub_pos,
                            'level': level_name,
                            'is_sub': True,
                            'distance': (sub_pos - camera_location).length,
                            'value': sub_coord,
                            'format': level_data['format'],
                            'interval': interval,
                            'group_distance': distance,  # 使用主刻度的距离
                            'transition_factor': transition_factor[level_name]  # 添加过渡因子
                        }
                        sub_points.append(sub_point)
                
                # 如果组内的点数量达到阈值，则添加整个组
                if len(sub_points) == len(SUB_DIVISIONS):
                    grid_group.extend(sub_points)
                    all_points.append(grid_group)

        # 对组进行排序（按组的距离）
        all_points.sort(key=lambda group: group[0]['group_distance'])

        # 处理重叠，保留较大刻度，同时保持组的完整性
        for group in all_points:
            main_point = group[0]
            pos_value = getattr(main_point['position'], axis.lower())
            
            # 检查主刻度是否有重叠
            has_overlap = False
            for used_pos in used_positions:
                if abs(pos_value - used_pos) < 0.001:
                    has_overlap = True
                    break
            
            # 如果没有重叠，添加整个组
            if not has_overlap:
                points.extend(group)
                used_positions.add(pos_value)

        return points

    def generate_ortho_grid_points(self, view_distance, axis, region, rv3d):
        """生成正交视图的网格点"""
        points = []
        
        # 获取视口边界
        bounds = self.get_ortho_view_bounds(region, rv3d)
        
        # 获取场景的网格比例和单位系统
        scene = bpy.context.scene
        grid_scale = scene.unit_settings.scale_length
        
        # 选择当前单位系统的层级
        current_system = 'metric' if self.unit_system == 'm' else 'imperial'
        current_levels = GRID_LEVELS[current_system]
        
        # 根据轴选择对应的边界
        if axis == 'X':
            min_bound = bounds['min_x']
            max_bound = bounds['max_x']
        elif axis == 'Y':
            min_bound = bounds['min_y']
            max_bound = bounds['max_y']
        else:  # Z轴
            min_bound = bounds['min_z']
            max_bound = bounds['max_z']
        
        # 计算最大绝对值，确保正负轴对称
        max_abs = max(abs(min_bound), abs(max_bound))
        min_bound = -max_abs
        max_bound = max_abs
        
        # 计算视图范围的大小
        range_size = max_bound - min_bound
        target_divisions = 10  # 期望的大致分段数（每边5个）
        
        # 根据视图范围选择合适的层级
        active_level = None
        best_division_diff = float('inf')
        transition_factor = 1.0  # 默认过渡因子为1（完全显示）
        
        for level_name, level_data in current_levels.items():
            # 应用网格比例
            scaled_interval = level_data['main_interval'] * grid_scale
            divisions = range_size / scaled_interval
            division_diff = abs(divisions - target_divisions)
            
            if division_diff < best_division_diff:
                best_division_diff = division_diff
                active_level = (level_name, level_data)
                # 计算过渡因子，确保层级切换时的平滑过渡
                transition_factor = max(0.1, min(1.0, 1.0 - division_diff / target_divisions))
        
        level_name, level_data = active_level
        base_interval = level_data['main_interval'] * grid_scale
        
        # 根据范围大小调整间隔
        interval = base_interval
        while range_size / interval > target_divisions * 2:
            interval *= 2
        while range_size / interval < target_divisions / 2:
            interval /= 2
        
        # 确保从0开始向两边扩展
        max_steps = math.ceil(max_abs / interval)
        min_coord = -max_steps * interval
        max_coord = max_steps * interval
        
        # 计算视图方向与轴的夹角
        view_vector = rv3d.view_rotation @ Vector((0, 0, -1))
        view_vector.normalize()
        axis_vector = Vector((1, 0, 0)) if axis == 'X' else \
                     Vector((0, 1, 0)) if axis == 'Y' else \
                     Vector((0, 0, 1))
        
        # 在正交视图下，我们只关心距离而不关心角度
        show_subdivisions = True  # 在正交视图下始终显示子刻度
        subdivision_opacity = 1.0  # 保持完全不透明
        
        # 生成主刻度点（包括正负轴）
        for coord in self.frange(min_coord, max_coord + interval, interval):
            pos = Vector((coord, 0, 0)) if axis == 'X' else \
                  Vector((0, coord, 0)) if axis == 'Y' else \
                  Vector((0, 0, coord))
            
            # 计算视口边缘淡出效果
            pos_2d = location_3d_to_region_2d(region, rv3d, pos)
            if pos_2d:
                margin = 50  # 边缘淡出区域的像素宽度
                edge_factor = min(
                    pos_2d.x / margin,
                    (region.width - pos_2d.x) / margin,
                    pos_2d.y / margin,
                    (region.height - pos_2d.y) / margin
                )
                edge_factor = max(0.0, min(1.0, edge_factor))
            else:
                edge_factor = 0.0
            
            # 主刻度点的不透明度由过渡因子和边缘因子共同控制
            main_point = {
                'position': pos,
                'level': level_name,
                'is_sub': False,
                'distance': view_distance,
                'value': coord,
                'format': level_data['format'],
                'interval': interval,
                'group_distance': view_distance,
                'transition_factor': transition_factor * edge_factor  # 结合边缘淡出效果
            }
            points.append(main_point)
            
            # 生成子刻度（除了最后一个主刻度）
            if show_subdivisions and coord < max_coord:
                next_coord = coord + interval
                
                # 显示所有子刻度
                sub_divisions = [0.25, 0.5, 0.75]
                
                for ratio in sub_divisions:
                    sub_coord = coord + (next_coord - coord) * ratio
                    if min_bound <= sub_coord <= max_bound:
                        sub_pos = Vector((sub_coord, 0, 0)) if axis == 'X' else \
                                Vector((0, sub_coord, 0)) if axis == 'Y' else \
                                Vector((0, 0, sub_coord))
                        
                        # 计算子刻度的视口边缘淡出效果
                        sub_pos_2d = location_3d_to_region_2d(region, rv3d, sub_pos)
                        if sub_pos_2d:
                            sub_edge_factor = min(
                                sub_pos_2d.x / margin,
                                (region.width - sub_pos_2d.x) / margin,
                                sub_pos_2d.y / margin,
                                (region.height - sub_pos_2d.y) / margin
                            )
                            sub_edge_factor = max(0.0, min(1.0, sub_edge_factor))
                        else:
                            sub_edge_factor = 0.0
                        
                        # 调整子刻度的透明度
                        sub_opacity = 0.85 if ratio == 0.7 else 0.7  # 中点刻度稍微明显一些，其他子刻度更淡
                        
                        sub_point = {
                            'position': sub_pos,
                            'level': level_name,
                            'is_sub': True,
                            'distance': view_distance,
                            'value': sub_coord,
                            'format': level_data['format'],
                            'interval': interval,
                            'group_distance': view_distance,
                            'transition_factor': transition_factor * sub_edge_factor * sub_opacity  # 结合边缘淡出效果和子刻度透明度
                        }
                        points.append(sub_point)
        
        # 在正交视图下为世界中心添加独立的0标注，并结合边缘淡出效果
        zero_pos = Vector((0, 0, 0))
        zero_pos_2d = location_3d_to_region_2d(region, rv3d, zero_pos)
        if zero_pos_2d:
            margin = 50  # 边缘淡出区域的像素宽度
            zero_edge_factor = min(
                zero_pos_2d.x / margin,
                (region.width - zero_pos_2d.x) / margin,
                zero_pos_2d.y / margin,
                (region.height - zero_pos_2d.y) / margin
            )
            zero_edge_factor = max(0.0, min(1.0, zero_edge_factor))
        else:
            zero_edge_factor = 0.0
        
        zero_point = {
            'position': zero_pos,
            'level': 'zero',
            'is_sub': False,
            'distance': view_distance,
            'value': 0.0,
            'format': "{:.1f}",
            'interval': interval,
            'group_distance': view_distance,
            'transition_factor': transition_factor * zero_edge_factor  # 结合边缘淡出效果
        }
        points.append(zero_point)
        
        return points

    def get_view_bounds(self, region, rv3d):
        """计算视口边界在世界空间中的范围"""
        from bpy_extras import view3d_utils
        
        # 获取视口的四个角点
        corners_2d = [
            Vector((0, 0)),  # 左下
            Vector((region.width, 0)),  # 右下
            Vector((region.width, region.height)),  # 右上
            Vector((0, region.height))  # 左上
        ]
        
        bounds = {'min_x': float('inf'), 'max_x': float('-inf'),
                 'min_y': float('inf'), 'max_y': float('-inf'),
                 'min_z': float('inf'), 'max_z': float('-inf')}
        
        camera_location = rv3d.view_matrix.inverted().translation
        
        for co2d in corners_2d:
            # 获取从相机到屏幕点的方向向量
            view_vector = view3d_utils.region_2d_to_vector_3d(region, rv3d, co2d)
            # 使用一个足够大的距离
            far_point = camera_location + view_vector * rv3d.view_distance * 10
            
            # 更新边界
            bounds['min_x'] = min(bounds['min_x'], camera_location.x, far_point.x)
            bounds['max_x'] = max(bounds['max_x'], camera_location.x, far_point.x)
            bounds['min_y'] = min(bounds['min_y'], camera_location.y, far_point.y)
            bounds['max_y'] = max(bounds['max_y'], camera_location.y, far_point.y)
            bounds['min_z'] = min(bounds['min_z'], camera_location.z, far_point.z)
            bounds['max_z'] = max(bounds['max_z'], camera_location.z, far_point.z)
        
        return bounds

    def get_ortho_view_bounds(self, region, rv3d):
        """计算正交视图的边界范围"""
        from bpy_extras import view3d_utils
        
        # 获取视口的四个角点
        corners_2d = [
            Vector((0, 0)),  # 左下
            Vector((region.width, 0)),  # 右下
            Vector((region.width, region.height)),  # 右上
            Vector((0, region.height))  # 左上
        ]
        
        bounds = {'min_x': float('inf'), 'max_x': float('-inf'),
                 'min_y': float('inf'), 'max_y': float('-inf'),
                 'min_z': float('inf'), 'max_z': float('-inf')}
        
        # 在正交视图中，使用view_matrix和view_distance计算实际边界
        view_matrix_inv = rv3d.view_matrix.inverted()
        view_location = view_matrix_inv.translation
        
        for co2d in corners_2d:
            # 获取从视口到3D空间的射线方向
            view_vector = view3d_utils.region_2d_to_vector_3d(region, rv3d, co2d)
            # 在正交视图中，所有点都在同一平面上
            # 使用view_distance来确定平面距离
            point_3d = view_location + view_vector * rv3d.view_distance
            
            # 更新边界
            bounds['min_x'] = min(bounds['min_x'], point_3d.x)
            bounds['max_x'] = max(bounds['max_x'], point_3d.x)
            bounds['min_y'] = min(bounds['min_y'], point_3d.y)
            bounds['max_y'] = max(bounds['max_y'], point_3d.y)
            bounds['min_z'] = min(bounds['min_z'], point_3d.z)
            bounds['max_z'] = max(bounds['max_z'], point_3d.z)
        
        # 扩展边界以确保完整覆盖
        margin = rv3d.view_distance * 0.5  # 使用视图距离的一半作为边界扩展
        bounds['min_x'] -= margin
        bounds['max_x'] += margin
        bounds['min_y'] -= margin
        bounds['max_y'] += margin
        bounds['min_z'] -= margin
        bounds['max_z'] += margin
        
        return bounds

    def frange(self, start, stop, step):
        """生成范围内的浮点数序列，类似于 range()"""
        while start <= stop:
            yield start
            start += step

    def smooth_value(self, current, last, memory_factor):
        if last is None:
            return current
        return last * memory_factor + current * (1 - memory_factor)

    def get_stable_transform(self, pos_2d, text):
        key = f"{text}_{int(pos_2d.x/20)}_{int(pos_2d.y/20)}"
        if key in self.last_positions['X'] or key in self.last_positions['Y'] or key in self.last_positions['Z']:
            try:
                if key in self.last_positions['X']:
                    last_pos = self.last_positions['X'].pop(key)
                elif key in self.last_positions['Y']:
                    last_pos = self.last_positions['Y'].pop(key)
                elif key in self.last_positions['Z']:
                    last_pos = self.last_positions['Z'].pop(key)
                pos_2d = Vector((
                    self.smooth_value(pos_2d.x, last_pos.x, self.position_memory),
                    self.smooth_value(pos_2d.y, last_pos.y, self.position_memory)
                ))
            except Exception as e:
                logger.exception(f"Error smoothing position for key: {key}, Error: {e}")
        # 根据轴类型决定存储在哪个缓存中
        axis = 'X' if 'x' in text.lower() else ('Y' if 'y' in text.lower() else 'Z')
        if axis == 'X':
            self.last_positions['X'][key] = pos_2d.copy()
            if len(self.last_positions['X']) > 800:
                removed_key, removed_value = self.last_positions['X'].popitem(last=False)
                logger.debug(f'Removed X axis cached position: {removed_key}')
        elif axis == 'Y':
            self.last_positions['Y'][key] = pos_2d.copy()
            if len(self.last_positions['Y']) > 800:
                removed_key, removed_value = self.last_positions['Y'].popitem(last=False)
                logger.debug(f'Removed Y axis cached position: {removed_key}')
        elif axis == 'Z':
            self.last_positions['Z'][key] = pos_2d.copy()
            if len(self.last_positions['Z']) > 800:
                removed_key, removed_value = self.last_positions['Z'].popitem(last=False)
                logger.debug(f'Removed Z axis cached position: {removed_key}')
        return pos_2d

    def get_unit_scale(self, text):
        try:
            # 使用正则表达式提取数字部分
            match = re.match(r'^-?(\d+(\.\d+)?)', text)
            if match:
                numeric_value = float(match.group(1))
                # 判断单位
                if 'cm' in text:
                    value = numeric_value / 100
                elif 'm' in text:
                    value = numeric_value
                elif 'in' in text:
                    value = numeric_value / 12  # 英寸转英尺
                elif 'ft' in text:
                    value = numeric_value
                else:
                    value = 1.0
            else:
                value = 1.0
        except (ValueError, IndexError) as e:
            logger.warning(f"Error parsing text '{text}': {e}")
            return 1.0
        abs_value = abs(value)
        if abs_value >= 1.0:
            return 1.2
        elif abs_value >= 0.5:
            return 1.1
        elif abs_value >= 0.1:
            return 1.0
        else:
            return 0.9

    def format_distance(self, value, is_sub=False, unit_system=None):
        """格式化距离值，根据当前单位系统进行转换"""
        if unit_system is None:
            unit_system = self.unit_system
        
        # 特殊处理0点
        if abs(value) < 1e-6:  # 使用小数点精度来判断是否为0
            return "0"  # 对0点使用简单的显示
        
        # 如果是英制单位，转换值
        if unit_system == 'ft':
            value = convert_to_imperial(value)
        
        # 根据大小选择合适的单位格式
        if unit_system == 'm':
            if abs(value) >= 1000:
                return f"{value/1000:.1f}km"
            else:
                return f"{value:.1f}m"
        else:  # 英制单位
            if abs(value) >= 5280:  # 1英里 = 5280英尺
                return f"{value/5280:.1f}mi"
            elif abs(value) >= 3:  # 3英尺以上用码
                return f"{value/3:.1f}yd"
            else:
                return f"{value:.0f}'"  # 英尺符号

    def draw_text(self, x, y, text, alpha, font_size, color, is_origin=False):
        """绘制文本，为原点添加特殊样式"""
        try:
            # 特殊处理原点标注
            if is_origin:
                # 原点使用稍大的字体和特殊颜色
                blf.size(0, int(font_size * 1.2))
                origin_color = (0.9, 0.9, 0.9)  # 接近白色
                blf.color(0, origin_color[0], origin_color[1], origin_color[2], alpha)
            else:
                blf.size(0, font_size)
                blf.color(0, color[0], color[1], color[2], alpha)
            
            # 获取文本尺寸以进行居中对齐
            text_width, text_height = blf.dimensions(0, text)
            
            # 调整位置使文本居中
            x = x - text_width / 2
            y = y + text_height / 2  # 向上偏移以给点标记留出空间
            
            # 为原点添加阴影效果
            if is_origin:
                blf.enable(0, blf.SHADOW)
                blf.shadow(0, 3, 0.0, 0.0, 0.0, alpha * 0.8)
            
            # 绘制文本
            blf.position(0, x, y, 0)
            blf.draw(0, text)
            
            if is_origin:
                blf.disable(0, blf.SHADOW)
            
            # 在数字下方绘制点标记
            if not is_origin:  # 原点不需要点标记
                dot_y = y - text_height - 2  # 点标记位置在文本下方
                blf.size(0, max(6, int(font_size * 0.5)))  # 点的大小
                blf.position(0, x + text_width/2 - 1, dot_y, 0)  # 居中对齐点标记
                blf.draw(0, "•")
                
        except Exception as e:
            logger.exception(f'Error drawing text: {e}')

    def get_view_orientation(self, rv3d):
        """获取当前视图方向，并处理旋转宽容度"""
        # 获取当前视图的旋转
        rotation = rv3d.view_rotation.copy()
        
        # 如果是用户视图，需要特殊处理
        if rv3d.view_perspective == 'ORTHO' and rv3d.is_orthographic_side_view:
            # 获取欧拉角
            euler = rotation.to_euler('XYZ')
            
            # 将角度转换到0-360度范围
            angles = [math.degrees(a) % 360 for a in euler]
            
            # 定义宽容度（度）
            tolerance = 5.0
            
            # 检查是否接近标准视图角度
            for i, angle in enumerate(angles):
                if abs(angle) < tolerance or abs(angle - 180) < tolerance:
                    if i == 0:  # X轴旋转
                        return 'TOP' if abs(angle) < tolerance else 'BOTTOM'
                    elif i == 1:  # Y轴旋转
                        return 'FRONT' if abs(angle) < tolerance else 'BACK'
                    elif i == 2:  # Z轴旋转
                        return 'RIGHT' if abs(angle) < tolerance else 'LEFT'
            
            # 如果不是标准视图，返回USER
            return 'USER'
        
        # 对于其他情况，使用四元数判断
        return self._get_orientation_from_rotation(rotation)

    def _get_orientation_from_rotation(self, rotation):
        """根据旋转四元数确定视图方向"""
        # 获取视图方向向量
        view_vector = rotation @ Vector((0, 0, -1))
        up_vector = rotation @ Vector((0, 1, 0))
        
        # 归一化向量
        view_vector.normalize()
        up_vector.normalize()
        
        # 设置阈值
        threshold = 0.9  # cos(约25度)
        
        # 判断主要朝向
        abs_view = [abs(v) for v in view_vector]
        max_index = abs_view.index(max(abs_view))
        
        if abs_view[max_index] > threshold:
            if max_index == 0:  # X轴
                return 'RIGHT' if view_vector[0] < 0 else 'LEFT'
            elif max_index == 1:  # Y轴
                return 'BACK' if view_vector[1] < 0 else 'FRONT'
            else:  # Z轴
                return 'TOP' if view_vector[2] < 0 else 'BOTTOM'
        
        return 'USER'

class VIEW3D_PT_grid_dimension_overlay(bpy.types.Panel):
    """UI面板：网格尺寸标注开关（视图叠加层）"""
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'HEADER'
    bl_label = _("Grid Dimension")
    bl_parent_id = "VIEW3D_PT_overlay"

    def draw(self, context):
        layout = self.layout
        layout.use_property_split = True
        layout.use_property_decorate = False
        
        row = layout.row()
        is_active = GridDimensionOverlay.is_overlay_active(context.area)

        row.operator(
            "view3d.grid_dimension_stop" if is_active else "view3d.grid_dimension_start", 
            text=_("Grid Dimension"), 
            depress=is_active,
            icon='OVERLAY' if is_active else 'GRID'
        )

        if is_active:
            box = layout.box()
            settings = context.scene.grid_dimension

            # 第一行：字体大小和颜色
            row = box.row()
            row.use_property_split = False
            row.alignment = 'EXPAND'
            
            split = row.split(factor=0.5)
            col = split.column()
            col.prop(settings, "font_size")
            
            col = split.column()
            col.prop(settings, "color", text="")
            
            # 第二行：Axis Color和Alpha
            row = box.row()
            row.use_property_split = False
            row.alignment = 'EXPAND'
            
            split = row.split(factor=0.5)
            col = split.column()
            col.prop(settings, "auto_color", slider=True)
            
            col = split.column()
            col.prop(settings, "alpha")

            # 第三行：Follow Scene Units、Show Z和Show Count
            row = box.row()
            row.use_property_split = False
            row.alignment = 'EXPAND'
            
            split = row.split(factor=0.33)
            col = split.column()
            col.prop(settings, "follow_scene_units")
            
            split = split.split(factor=0.5)
            col = split.column()
            col.prop(settings, "show_z")
            
            col = split.column()
            col.prop(settings, "show_count")

            if not settings.follow_scene_units:
                box.prop(settings, "unit_system")

            overlay = GridDimensionOverlay.overlays.get(context.area.as_pointer())
            if overlay:
                overlay.update_settings(context.scene)

class GridDimensionStartOperator(bpy.types.Operator):
    """启动网格尺寸标注"""
    bl_idname = "view3d.grid_dimension_start"
    bl_label = _("Start Grid Dimension")
    bl_description = _("Show grid dimension annotations")
    bl_options = {'REGISTER'}

    def execute(self, context):
        if context.area.type != 'VIEW_3D':
            self.report({'WARNING'}, _("Current area is not a 3D View."))
            return {'CANCELLED'}
        if GridDimensionOverlay.is_overlay_active(context.area):
            self.report({'INFO'}, _("Grid Dimension overlay is already active."))
            return {'CANCELLED'}
        overlay = GridDimensionOverlay(context.area)
        overlay.register(self.report)
        return {'FINISHED'}

class GridDimensionStopOperator(bpy.types.Operator):
    """停止网格尺寸标注"""
    bl_idname = "view3d.grid_dimension_stop"
    bl_label = _("Stop Grid Dimension")
    bl_description = _("Hide grid dimension annotations")
    bl_options = {'REGISTER'}

    def execute(self, context):
        if context.area.type != 'VIEW_3D':
            self.report({'WARNING'}, _("Current area is not a 3D View."))
            return {'CANCELLED'}
        overlay = GridDimensionOverlay.overlays.get(context.area.as_pointer())
        if overlay:
            overlay.unregister()
            self.report({'INFO'}, _("Grid Dimension overlay has been stopped."))
        else:
            self.report({'WARNING'}, _("Grid Dimension overlay is not active."))
        return {'FINISHED'}

def update_overlay():
    """
    更新所有激活的覆盖层设置。
    """
    for overlay in GridDimensionOverlay.overlays.values():
        overlay.update_settings(bpy.context.scene)

def unit_system_update_handler(scene):
    """
    处理场景单位系统变化的处理器。
    如果`follow_scene_units`启用，自动同步插件单位系统。
    """
    global previous_unit_system
    current_system = scene.unit_settings.system
    scene_pointer = scene.as_pointer()
    
    # 初始化之前的单位系统
    if scene_pointer not in previous_unit_system:
        previous_unit_system[scene_pointer] = current_system
    
    # 检查单位系统是否变化
    if previous_unit_system[scene_pointer] != current_system:
        previous_unit_system[scene_pointer] = current_system
        settings = scene.grid_dimension  # 直接访问 PropertyGroup
        if settings.follow_scene_units:
            # 自动同步插件的单位系统
            if current_system == 'IMPERIAL':
                settings.unit_system = 'ft'
                logger.info(f"Switched to imperial units (feet)")
            else:
                settings.unit_system = 'm'
                logger.info(f"Switched to metric units (meters)")
            logger.info(f"Auto-synced grid_dimension_unit_system to '{settings.unit_system}' based on scene unit system '{current_system}'.")
            update_overlay()

def subscribe_msgbus():
    """
    使用 bpy.msgbus 订阅属性变化。
    """
    # 订阅 PropertyGroup 中的所有相关属性
    bpy.msgbus.subscribe_rna(
        key=(bpy.types.Scene, "grid_dimension"),
        owner=GRID_DIMENSION_OWNER,
        args=(),
        notify=update_overlay
    )

    # 订阅 UnitSettings 的 system 属性变化
    bpy.msgbus.subscribe_rna(
        key=(bpy.types.UnitSettings, "system"),
        owner=GRID_DIMENSION_OWNER,
        args=(),
        notify=lambda: unit_system_update_handler(bpy.context.scene)
    )

def unsubscribe_msgbus():
    """
    取消所有 msgbus 的订阅。
    """
    bpy.msgbus.clear_by_owner(GRID_DIMENSION_OWNER)

def register():
    # 注册属性组
    bpy.utils.register_class(GridDimensionSettings)
    bpy.types.Scene.grid_dimension = bpy.props.PointerProperty(type=GridDimensionSettings)
    
    # 注册操作符和面板
    bpy.utils.register_class(GridDimensionStartOperator)
    bpy.utils.register_class(GridDimensionStopOperator)
    bpy.utils.register_class(VIEW3D_PT_grid_dimension_overlay)
    
    # 订阅消息总线
    subscribe_msgbus()
    
    # 确保在注册时更新翻译
    bpy.app.translations.register(__name__, translations_dict)

def unregister():
    # 取消订阅消息总线
    unsubscribe_msgbus()
    
    # 注销操作符和面板
    bpy.utils.unregister_class(VIEW3D_PT_grid_dimension_overlay)
    bpy.utils.unregister_class(GridDimensionStopOperator)
    bpy.utils.unregister_class(GridDimensionStartOperator)
    
    # 注销属性组
    del bpy.types.Scene.grid_dimension
    bpy.utils.unregister_class(GridDimensionSettings)
    
    # 注销翻译
    bpy.app.translations.unregister(__name__)

# 定义空的翻译字典，实际翻译将通过Blender的翻译系统处理
translations_dict = {}

if __name__ == "__main__":
    register()